@@ -25,6 +25,18 @@ __all__ = [
 ]
 
 
+def strip_nested_dimensions(expr):
+    """
+    Strip dimensions from array expressions of arbitrary derived-type
+    nesting depth.
+    """
+
+    parent = expr.parent
+    if parent:
+        parent = strip_nested_dimensions(parent)
+    return expr.clone(dimensions=None, parent=parent)
+
+
 class DataflowAnalysisAttacher(Transformer):
     """
     Analyse and attach in-place the definition, use and live status of
@@ -62,9 +74,12 @@ class DataflowAnalysisAttacher(Transformer):
         """
         Return set of symbols found in an expression.
         """
+        variables = {strip_nested_dimensions(v) for v in FindVariables().visit(expr)}
+        parents = {p for var in variables for p in var.parents}
+        variables -= parents
         if condition is not None:
-            return {v.clone(dimensions=None) for v in FindVariables().visit(expr) if condition(v)}
-        return {v.clone(dimensions=None) for v in FindVariables().visit(expr)}
+            return {v for v in variables if condition(v)}
+        return variables
 
     @classmethod
     def _symbols_from_lhs_expr(cls, expr):
@@ -81,7 +96,7 @@ class DataflowAnalysisAttacher(Transformer):
         (defines, uses) : (set, set)
             The sets of defined and used symbols (in that order).
         """
-        defines = {expr.clone(dimensions=None)}
+        defines = {strip_nested_dimensions(expr)}
         uses = cls._symbols_from_expr(getattr(expr, 'dimensions', ()))
         return defines, uses
 
@@ -134,7 +149,10 @@ class DataflowAnalysisAttacher(Transformer):
     def visit_Loop(self, o, **kwargs):
         # A loop defines the induction variable for its body before entering it
         live = kwargs.pop('live_symbols', set())
+        mem_calls = as_tuple(i for i in FindInlineCalls().visit(o.bounds) if i.function in self._mem_property_queries)
+        query_args = as_tuple(flatten(FindVariables().visit(i.parameters) for i in mem_calls))
         uses = self._symbols_from_expr(o.bounds)
+        uses = {v for v in uses if not v in query_args}
         body, defines, uses = self._visit_body(o.body, live=live|{o.variable.clone()}, uses=uses, **kwargs)
         o._update(body=body)
         # Make sure the induction variable is not considered outside the loop
@@ -188,6 +206,8 @@ class DataflowAnalysisAttacher(Transformer):
         defines = defines | else_defines
         return self.visit_Node(o, live_symbols=live, defines_symbols=defines, uses_symbols=uses, **kwargs)
 
+    visit_TypeConditional = visit_MultiConditional
+
     def visit_MaskedStatement(self, o, **kwargs):
         live = kwargs.pop('live_symbols', set())
         conditions = self._symbols_from_expr(o.conditions)
