@@ -18,12 +18,12 @@ from loki.types import SymbolAttributes, BasicType, DerivedType
 from loki.expression import (
     symbols as sym, Variable, Array, RangeIndex
 )
+from loki.transformations.array_indexing import resolve_vector_dimension
 from loki.transformations.sanitise import do_resolve_associates
 from loki.transformations.utilities import (
     recursive_expression_map_update, get_integer_variable,
-    get_loop_bounds, check_routine_sequential
+    check_routine_sequential
 )
-from loki.transformations.single_column.base import SCCBaseTransformation
 
 __all__ = ['BlockViewToFieldViewTransformation', 'InjectBlockIndexTransformation',
         'LowerBlockIndexTransformation', 'LowerBlockLoopTransformation']
@@ -256,15 +256,12 @@ class BlockViewToFieldViewTransformation(Transformation):
 
         # Sanitize the subroutine
         do_resolve_associates(routine)
-        v_index = get_integer_variable(routine, name=self.horizontal.index)
-        SCCBaseTransformation.resolve_masked_stmts(routine, loop_variable=v_index)
 
         # Bail if routine is marked as sequential
         if check_routine_sequential(routine):
             return
 
-        bounds = get_loop_bounds(routine, self.horizontal)
-        SCCBaseTransformation.resolve_vector_dimension(routine, loop_variable=v_index, bounds=bounds)
+        resolve_vector_dimension(routine, dimension=self.horizontal)
 
         # for kernels we process the entire body
         routine.body = self.process_body(routine.body, item, successors, targets, exclude_arrays)
