@@ -15,6 +15,8 @@ from pathlib import Path
 from loki.batch.transformation import Transformation
 from loki.logging import debug
 
+__all__ = ['CMakePlanTransformation']
+
 class CMakePlanTransformation(Transformation):
     """
     Gather the planning information from all :any:`Item.trafo_data` to which
@@ -67,9 +69,9 @@ class CMakePlanTransformation(Transformation):
 
     def __init__(self, rootpath=None):
         self.rootpath = None if rootpath is None else Path(rootpath).resolve()
-        self.sources_to_append = []
-        self.sources_to_remove = []
-        self.sources_to_transform = []
+        self.sources_to_append = {}
+        self.sources_to_remove = {}
+        self.sources_to_transform = {}
 
     def plan_file(self, sourcefile, **kwargs):
         item = kwargs.get('item')
@@ -79,7 +81,7 @@ class CMakePlanTransformation(Transformation):
         if not 'FileWriteTransformation' in item.trafo_data:
             return
 
-        sourcepath = item.path.resolve()
+        sourcepath = item.path
 
         # This makes sure the sourcepath does in fact exist. Combined with
         # item duplication or other transformations we might end up adding
@@ -88,34 +90,72 @@ class CMakePlanTransformation(Transformation):
         source_exists = sourcepath.exists()
 
         if self.rootpath is not None:
-            sourcepath = sourcepath.relative_to(self.rootpath)
+            sourcepath = sourcepath.resolve().relative_to(self.rootpath)
 
         newsource = item.trafo_data['FileWriteTransformation']['path']
 
         debug(f'Planning:: {item.name} (role={item.role}, mode={item.mode})')
 
+        key = item.lib
         if newsource not in self.sources_to_append:
             if source_exists:
-                self.sources_to_transform += [sourcepath]
+                self.sources_to_transform.setdefault(key,[]).append(sourcepath)
             if item.replicate:
                 # Add new source file next to the old one
-                self.sources_to_append += [newsource]
+                self.sources_to_append.setdefault(key,[]).append(newsource)
             else:
                 # Replace old source file to avoid ghosting
-                self.sources_to_append += [newsource]
+                self.sources_to_append.setdefault(key,[]).append(newsource)
                 if source_exists:
-                    self.sources_to_remove += [sourcepath]
+                    if self.rootpath is not None:
+                        self.sources_to_remove.setdefault(key,[]).append(sourcepath)
+                    else:
+                        # NB: we use the item path directly here instead of resolving it,
+                        #     to stay compatible with what has been provided on the CLI.
+                        #     This is because the build system will likely use this path
+                        #     internally to identify the original file, and if the paths
+                        #     don't match the removal of source files from the target fails.
+                        self.sources_to_remove.setdefault(key,[]).append(item.path)
+
+    def _write_plan(self, filepath):
+        """
+        Write the key/target/library independent part of CMake plan file to :data:`filepath`
+        """
+        sources_to_transform = [s for sources in self.sources_to_transform.values() for s in sources]
+        sources_to_append = [s for sources in self.sources_to_append.values() for s in sources]
+        sources_to_remove = [s for sources in self.sources_to_remove.values() for s in sources]
+
+        with Path(filepath).open('w') as f:
+            s_transform = '\n'.join(f'    {s}' for s in sources_to_transform)
+            f.write(f'set( LOKI_SOURCES_TO_TRANSFORM \n{s_transform}\n   )\n')
+
+            s_append = '\n'.join(f'    {s}' for s in sources_to_append)
+            f.write(f'set( LOKI_SOURCES_TO_APPEND \n{s_append}\n   )\n')
+
+            s_remove = '\n'.join(f'    {s}' for s in sources_to_remove)
+            f.write(f'set( LOKI_SOURCES_TO_REMOVE \n{s_remove}\n   )\n')
 
     def write_plan(self, filepath):
         """
         Write the CMake plan file to :data:`filepath`
         """
-        with Path(filepath).open('w') as f:
-            s_transform = '\n'.join(f'    {s}' for s in self.sources_to_transform)
-            f.write(f'set( LOKI_SOURCES_TO_TRANSFORM \n{s_transform}\n   )\n')
+        # write plan disregarding the key/target/library
+        self._write_plan(filepath)
 
-            s_append = '\n'.join(f'    {s}' for s in self.sources_to_append)
-            f.write(f'set( LOKI_SOURCES_TO_APPEND \n{s_append}\n   )\n')
+        # write plan file for each key/target/library
+        keys = set(self.sources_to_transform.keys()) | \
+                set(self.sources_to_append.keys()) | set(self.sources_to_remove.keys())
+        for key in keys:
+            if key is None:
+                continue
+            with Path(filepath).open('a') as f:
+                # sanitize key = target, e.g., remove '.' and replace with '_'
+                sanitized_key = key.replace('.', '_')
+                s_transform = '\n'.join(f'    {s}' for s in self.sources_to_transform.get(key, ()))
+                f.write(f'set( LOKI_SOURCES_TO_TRANSFORM_{sanitized_key} \n{s_transform}\n   )\n')
 
-            s_remove = '\n'.join(f'    {s}' for s in self.sources_to_remove)
-            f.write(f'set( LOKI_SOURCES_TO_REMOVE \n{s_remove}\n   )\n')
+                s_append = '\n'.join(f'    {s}' for s in self.sources_to_append.get(key, ()))
+                f.write(f'set( LOKI_SOURCES_TO_APPEND_{sanitized_key} \n{s_append}\n   )\n')
+
+                s_remove = '\n'.join(f'    {s}' for s in self.sources_to_remove.get(key, ()))
+                f.write(f'set( LOKI_SOURCES_TO_REMOVE_{sanitized_key} \n{s_remove}\n   )\n')
