@@ -81,7 +81,7 @@ module type_def_mod
 
    type :: geom_type
       type(geom_dims) :: dim
-      integer, allocatable :: metadata(:)
+      integer, pointer :: metadata(:)
    end type
 end module type_def_mod
             """.strip()
@@ -334,7 +334,7 @@ def test_offload_deepcopy_analysis(frontend, config, deepcopy_code, expected_ana
 
         # check that the warning for pointer associations is produced
         messages = [log.message for log in caplog.records]
-        assert '[Loki::DataOffloadDeepcopyAnalysis] Pointer associations found in kernel' in messages
+        assert '[Loki::DataOffloadDeepcopyAnalysis] Pointer associations found in kernel' in messages[-1]
 
     # The analysis is tied to driver loops
     trafo_data_key = transformation._key
@@ -482,6 +482,8 @@ def check_geometry(conds, pragmas, routine):
     conds = [c for c in conds
              if 'geometry%metadata' in c.condition.parameters]
 
+    assert all(c.condition.name.lower() == 'associated' for c in conds)
+
     # geometry should only have copy and wipe related instructions
     assert len(conds) == 2
 
