@@ -5,6 +5,8 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 
+# pylint: disable=too-many-lines
+
 import pytest
 
 from loki import Subroutine, Sourcefile, Dimension, fgen
@@ -101,55 +103,69 @@ def test_scc_base_resolve_vector_notation(frontend, horizontal):
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_base_masked_statement(frontend, horizontal):
+@pytest.mark.parametrize('rel_index', ('jl', 'jcol', 'ji'))
+@pytest.mark.parametrize('indices', (('jl', 'jcol', 'jlll'), ('jcol','jcol', 'jcol'),
+    ('jl', 'jl', 'jl'), ('jcol', 'jlll', 'jlll')))
+def test_scc_base_rename_index_aliases(frontend, rel_index, indices):
     """
-    Test resolving of masked statements in kernel.
+    Test rename index aliases in kernel.
     """
 
-    fcode_kernel = """
-  SUBROUTINE compute_column(start, end, nlon, nz, q, t)
+    fcode_kernel = f"""
+  SUBROUTINE kernel_rename_index_aliases(start, end, nlon, nz, q, t)
     INTEGER, INTENT(IN) :: start, end  ! Iteration indices
     INTEGER, INTENT(IN) :: nlon, nz    ! Size of the horizontal and vertical
     REAL, INTENT(INOUT) :: t(nlon,nz)
     REAL, INTENT(INOUT) :: q(nlon,nz)
-    INTEGER :: jk
+    INTEGER :: jk {', jl' if 'jl' in indices else ''}
+    {'INTEGER :: jcol' if 'jcol' in indices else ''}
+    {'INTEGER :: jlll' if 'jlll' in indices else ''}
     REAL :: c
 
     c = 5.345
     DO jk = 2, nz
-      WHERE (q(start:end, jk) > 1.234)
-        q(start:end, jk) = q(start:end, jk-1) + t(start:end, jk) * c
-      ELSEWHERE
-        q(start:end, jk) = t(start:end, jk)
-      END WHERE
+      DO {indices[0]} = start, end
+        t({indices[0]}, jk) = c * jk
+        q({indices[0]}, jk) = q({indices[0]}, jk-1) + t({indices[0]}, jk) * c
+      END DO
     END DO
-  END SUBROUTINE compute_column
+    DO jk = 2, nz
+      DO {indices[1]} = start, end
+        t({indices[1]}, jk) = c * jk
+        q({indices[1]}, jk) = q({indices[1]}, jk-1) + t({indices[1]}, jk) * c
+      END DO
+    END DO
+    DO jk = 2, nz
+      DO {indices[2]} = start, end
+        t({indices[2]}, jk) = c * jk
+        q({indices[2]}, jk) = q({indices[2]}, jk-1) + t({indices[2]}, jk) * c
+      END DO
+    END DO
+  END SUBROUTINE kernel_rename_index_aliases
 """
+
     kernel = Subroutine.from_source(fcode_kernel, frontend=frontend)
 
+    h_indices = [rel_index] + [_index for _index in indices if _index != rel_index]
+    horizontal = Dimension(
+        name='horizontal', size=['dims%klon', 'nlon'], index=h_indices,
+        aliases=('nproma',), lower=('START', 'dims%ist'),
+        upper=('end', 'dims%iend')
+    )
     scc_transform = SCCBaseTransformation(horizontal=horizontal)
-    scc_transform.apply(kernel, role='kernel')
+    scc_transform.apply(kernel, role='kernel', rename_index_aliases=True)
 
-    # Ensure horizontal loop variable has been declared
-    assert 'jl' in kernel.variables
+    # ensure correct horizontal index is within the kernel variables
+    assert rel_index in kernel.variables
+    # ensure no horizontal index alias is in the kernel variables
+    for h_index in h_indices[1:]:
+        assert h_index not in kernel.variables
 
-    # Ensure we have three loops in the kernel,
-    # horizontal loops should be nested within vertical
-    kernel_loops = FindNodes(Loop).visit(kernel.body)
-    assert len(kernel_loops) == 2
-    assert kernel_loops[1] in FindNodes(Loop).visit(kernel_loops[0].body)
-    assert kernel_loops[1].variable == 'jl'
-    assert kernel_loops[1].bounds == 'start:end'
-    assert kernel_loops[0].variable == 'jk'
-    assert kernel_loops[0].bounds == '2:nz'
-
-    # Ensure that the respective conditional has been inserted correctly
-    kernel_conds = FindNodes(Conditional).visit(kernel.body)
-    assert len(kernel_conds) == 1
-    assert kernel_conds[0] in FindNodes(Conditional).visit(kernel_loops[1])
-    assert kernel_conds[0].condition == 'q(jl, jk) > 1.234'
-    assert fgen(kernel_conds[0].body) == 'q(jl, jk) = q(jl, jk - 1) + t(jl, jk)*c'
-    assert fgen(kernel_conds[0].else_body) == 'q(jl, jk) = t(jl, jk)'
+    # make sure all the relevant loops are there and have the correct variable
+    loops = FindNodes(Loop).visit(kernel.body)
+    h_loops = [loop for loop in loops if loop.bounds.lower == 'start']
+    assert len(h_loops) == 3
+    assert all(h_loop.variable == rel_index for h_loop in h_loops)
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -1122,11 +1138,11 @@ def test_scc_multiple_acc_pragmas(frontend, horizontal, blocking, dims_type, tmp
         with caplog.at_level(WARNING):
             scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
         if frontend == OMNI:
+            assert len(caplog.records) == 3
+            message = caplog.records[2].message
+        else:
             assert len(caplog.records) == 2
             message = caplog.records[1].message
-        else:
-            assert len(caplog.records) == 1
-            message = caplog.records[0].message
         assert "[Loki-SCC::Annotate] dynamically allocated structs are being privatised: ['local_dims']" in message
     else:
         scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
@@ -1162,6 +1178,77 @@ def test_scc_multiple_acc_pragmas(frontend, horizontal, blocking, dims_type, tmp
     assert len(loops) == 2
 
 
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_scc_driver_loop_async(frontend, horizontal, blocking):
+    """
+    Test that an annotated async driver loop will be mapped to an async acc pragma
+    """
+
+    fcode = """
+    subroutine test(work, nlon, nb)
+    implicit none
+
+      integer, intent(in) :: nb, nlon
+      real, dimension(nlon, nb), intent(inout) :: work
+      integer :: b, queue
+
+      queue = 0
+
+      !$loki data async(queue)
+      !$loki driver-loop async(queue)
+        do b=1, nb
+           call some_kernel(nlon, work(:,b))
+        enddo
+      !$loki end data
+
+    end subroutine test
+
+    subroutine some_kernel(nlon, work)
+    implicit none
+
+      integer, intent(in) :: nlon
+      real, dimension(nlon), intent(inout) :: work
+      integer :: jl
+
+      do jl=1,nlon
+         work(jl) = work(jl) + 1.
+      enddo
+
+    end subroutine some_kernel
+    """
+
+    source = Sourcefile.from_source(fcode, frontend=frontend)
+    routine = source['test']
+    routine.enrich(source.all_subroutines)
+
+    data_offload = DataOffloadTransformation(remove_openmp=True)
+    data_offload.transform_subroutine(routine, role='driver', targets=['some_kernel',])
+    pragma_model = PragmaModelTransformation(directive='openacc')
+    pragma_model.transform_subroutine(routine, role='driver', targets=['some_kernel',])
+
+    scc_pipeline = SCCVectorPipeline(
+        horizontal=horizontal, block_dim=blocking, directive='openacc'
+    )
+    scc_pipeline.apply(routine, role='driver', targets=['some_kernel',])
+
+    # Check that both acc pragmas are created
+    pragmas = FindNodes(Pragma).visit(routine.ir)
+    assert len(pragmas) == 4
+    assert pragmas[0].keyword == 'acc'
+    assert pragmas[1].keyword == 'acc'
+    assert pragmas[2].keyword == 'acc'
+    assert pragmas[3].keyword == 'acc'
+
+    assert 'data' in pragmas[0].content
+    assert 'copy' in pragmas[0].content
+    assert '(work)' in pragmas[0].content
+    assert 'async(queue)' in pragmas[0].content
+
+    assert pragmas[1].content == 'parallel loop gang vector_length(nlon) async(queue)'
+    assert pragmas[2].content == 'end parallel loop'
+    assert pragmas[3].content == 'end data'
+
+
 @pytest.mark.parametrize('frontend', available_frontends())
 def test_scc_annotate_routine_seq_pragma(frontend, blocking):
     """
@@ -1532,37 +1619,98 @@ def test_scc_inline_and_sequence_association(
         assert fgen(assign).lower() == 'work(jl) = 1.'
 
 
-@pytest.mark.xfail(not HAVE_FP, reason="Identification of array reduction intrinsics requires fparser.")
 @pytest.mark.parametrize('frontend', available_frontends())
-def test_scc_resolve_vector_dimension(frontend, horizontal):
+@pytest.mark.parametrize('pipeline,mode', [
+    (SCCVVectorPipeline, 'vector'),
+    (SCCSVectorPipeline, 'seq')
+])
+def test_scc_annotate_driver_loop(frontend, tmp_path, pipeline, mode):
     """
-    Test SCC vector resolution utility.
+    Test for issue #246 to ensure the correct loop in the loop nest
+    is identified and annotated as driver loop
     """
-
     fcode = """
-subroutine kernel(start, end, nlon, work)
-   integer, intent(in) :: start, end, nlon
-   real, intent(out) :: work(nlon)
-   integer :: jl
-   real :: work_maxval
+module mod
+    implicit none
+    integer, parameter :: ngpblk = 2
+    integer, parameter :: klon = 10
+    integer, parameter :: klev = 5
+    contains
+    subroutine wrapper(arr)
+        integer, parameter :: niter = 10
+        integer, intent(in) :: arr(klon, klev, ngpblk)
+        integer :: jblk, jiter
 
-   work(start:end) = 0.
-   work_maxval = maxval(work(start:end))
+        ! Begin relevant part.
+        DO jiter=1,niter
+            DO jblk=1,ngpblk
+                CALL kernel(klon, klev, 1, 1, arr(:, :, jblk))
+            END DO
+        END DO
+        ! End relevant part.
 
-end subroutine kernel
-"""
+    end subroutine wrapper
 
-    routine = Subroutine.from_source(fcode, frontend=frontend)
-    SCCBaseTransformation.resolve_vector_dimension(routine, routine.variable_map[horizontal.index],
-                                                   horizontal.bounds)
+    subroutine kernel(klon, klev, jlev_lower, jlon_lower, arr)
+        integer, intent(in) :: klon
+        integer, intent(in) :: klev
+        integer, intent(in) :: jlev_lower, jlon_lower
+        integer, intent(inout) :: arr(klon, klev)
+        integer :: jlon, jlev
 
-    loops = FindNodes(Loop).visit(routine.body)
-    assigns = FindNodes(Assignment).visit(routine.body)
-    assert len(assigns) == 2
+        DO jlev=jlev_lower,klev
+            DO jlon=jlon_lower,klon
+                arr(jlon, jlev) = jlon + jlev
+            END DO
+        END DO
 
-    assert len(loops) == 1
-    assert assigns[0] in loops[0].body
-    assert not assigns[1] in loops[0].body
+    end subroutine kernel
 
-    assert 'maxval' == assigns[1].rhs.name.lower()
-    assert 'start:end' in assigns[1].rhs.parameters[0].dimensions
+end module mod
+    """.strip()
+
+    source = Sourcefile.from_source(fcode, frontend=frontend, xmods=(tmp_path,))
+
+    horizontal = Dimension('horizontal', index='JLON', bounds=['jlon_lower', 'klon'], size='KLON')
+    block_dim = Dimension('block_dim', index='JBLK', size='NGPBLK')
+
+    # Test OpenACC annotations on non-hoisted version
+    for transform in pipeline(horizontal=horizontal, block_dim=block_dim, directive='openacc').transformations:
+        transform.apply(source['wrapper'], role='driver', targets=['kernel'])
+        transform.apply(source['kernel'], role='kernel')
+
+    # Ensure routine is annotated at vector level
+    pragmas = FindNodes(Pragma).visit(source['kernel'].ir)
+    if mode == 'vector':
+        assert len(pragmas) == 5
+    else:
+        assert len(pragmas) == 4
+    assert pragmas[0].keyword == 'acc'
+    assert pragmas[0].content == f'routine {mode}'
+    assert pragmas[1].keyword == 'acc'
+    assert pragmas[1].content == 'data present(arr)'
+    assert pragmas[-1].keyword == 'acc'
+    assert pragmas[-1].content == 'end data'
+
+    # Ensure vector and seq loops are annotated, including privatized variable `b`
+    with pragmas_attached(source['kernel'], Loop):
+        kernel_loops = FindNodes(Loop).visit(source['kernel'].ir)
+        assert kernel_loops[0].pragma[-1].keyword == 'acc'
+        assert kernel_loops[0].pragma[-1].content == f'loop {mode}'
+        if mode == 'vector':
+            assert len(kernel_loops) == 2
+            assert kernel_loops[1].pragma[-1].keyword == 'acc'
+            assert kernel_loops[1].pragma[-1].content == 'loop seq'
+        else:
+            assert len(kernel_loops) == 1
+
+    # Ensure a single outer parallel loop in driver
+    with pragmas_attached(source['wrapper'], Loop):
+        driver_loops = FindNodes(Loop).visit(source['wrapper'].body)
+        if mode == 'vector':
+            assert len(driver_loops) == 2
+        else:
+            assert len(driver_loops) == 3
+        assert not driver_loops[0].pragma
+        assert driver_loops[1].pragma[-1].keyword == 'acc'
+        assert driver_loops[1].pragma[-1].content == 'parallel loop gang'
